From ef70c39d52a3aa53e95beb64f877da78daffb0f4 Mon Sep 17 00:00:00 2001
From: Yuan Fu <casouri@gmail.com>
Date: Thu, 15 Oct 2020 22:20:20 -0400
Subject: [PATCH] Suppress "Before first headings" error in outline-cycle

* lisp/outline.el (outline-cycle, outline-cycle-buffer): Suppress error.
---
 lisp/outline.el | 41 ++++++++++++++++++++++-------------------
 1 file changed, 22 insertions(+), 19 deletions(-)

diff --git a/lisp/outline.el b/lisp/outline.el
index a4ce9afb44..5e22850292 100644
--- a/lisp/outline.el
+++ b/lisp/outline.el
@@ -1167,19 +1167,22 @@ outline-cycle
 `Headings only' means show sub headings but not their bodies.
 `Show all' means show all subheadings and their bodies."
   (interactive)
-  (pcase (outline--cycle-state)
-    ('hide-all
-     (if (outline-has-subheading-p)
-         (progn (outline-show-children)
-                (message "Only headings"))
-       (outline-show-subtree)
-       (message "Show all")))
-    ('headings-only
-     (outline-show-subtree)
-     (message "Show all"))
-    ('show-all
-     (outline-hide-subtree)
-     (message "Hide all"))))
+  (condition-case nil
+      (pcase (outline--cycle-state)
+        ('hide-all
+         (if (outline-has-subheading-p)
+             (progn (outline-show-children)
+                    (message "Only headings"))
+           (outline-show-subtree)
+           (message "Show all")))
+        ('headings-only
+         (outline-show-subtree)
+         (message "Show all"))
+        ('show-all
+         (outline-hide-subtree)
+         (message "Hide all")))
+    ;; If error: "Before first heading" occurs, ignore it.
+    (error nil)))
 
 (defvar-local outline--cycle-buffer-state 'show-all
   "Internal variable used for tracking buffer cycle state.")
@@ -1190,12 +1193,12 @@ outline-cycle-buffer
   (pcase outline--cycle-buffer-state
     ('show-all
      (save-excursion
-       (let ((start-point (point)))
-         (while (not (eq (point) start-point))
-           (outline-up-heading 1))
-         (outline-hide-sublevels
-          (progn (outline-back-to-heading)
-                 (funcall 'outline-level)))))
+       (outline-hide-sublevels
+        ;; If error: "Before first heading" occurs, ignore it.
+        (or (ignore-errors
+              (outline-back-to-heading)
+              (outline-level))
+            1)))
      (setq outline--cycle-buffer-state 'top-level)
      (message "Top level headings"))
     ('top-level
-- 
2.27.0

